<?php
namespace App\Factory;

use App\Exception\ParamsException as ParamsException;

class ProductFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util       = $util;
    }
    
    public function getItensVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda) {
        
        $sql = "SELECT I.NRSEQUITVEND, I.NRSEQVENDA, P.NMPRODUTO, I.CDPRODUTO, P.CDARVPROD, P.CDPRODINTE,
                       I.QTPRODVEND, I.VRUNITVEND, (I.VRUNITVEND * I.QTPRODVEND) AS TOTAL , I.VRDESITVEND,
                       I.VRACRITVEND, P.CDCLASFISC, P.CDCEST, II.CDCSTPRODI, II.CDCSTPRODPC, II.CDCFOPPROD,
                       II.VRBCPISCOFINS, II.VRIMPPIS, II.VRIMPCOFINS, II.VRBASECALCICMS, II.VRIMPOPRODIT,
                       A.VRALIQPIS, A.VRALIQCOFINS, A.VRPEALIMPFIS, I.VRUNITVENDCL, I.CDCAIXAPEDVEN
                FROM ITEMVENDA I
                    JOIN PRODUTO P
                        ON  I.CDPRODUTO = P.CDPRODUTO
                    JOIN ALIQIMPFIS A
                        ON I.CDPRODUTO = A.CDPRODUTO
                        AND I.CDFILIAL = A.CDFILIAL
                    JOIN ITVENDAIMPOS II
                        ON I.NRSEQUITVEND = II.NRSEQUITVEND
                        AND I.CDFILIAL   = II.CDFILIAL
                        AND I.NRSEQVENDA = II.NRSEQVENDA
                        AND I.CDCAIXA    = II.CDCAIXA
                WHERE I.CDFILIAL = :CDFILIAL
                    AND I.CDCAIXA    = :CDCAIXA
                    AND I.NRSEQVENDA = :NRSEQVENDA
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
        
        $itens = $this->connection->fetchAll($sql, $params);
        
        return $itens;
    }
    
    public function getItens($subsidiaryCode, $dtInicial, $dtFinal) {
        
        $sql = "SELECT IV.CDFILIAL, IV.CDCAIXA, IV.NRSEQVENDA, IV.NRSEQUITVEND, IV.QTPRODVEND, 
                       IV.CDPRODUTO, P.NMPRODUTO, IV.DTHRPEDIDO, P.IDTIPOCOMPPROD, P.IDUTILRETPRODACAB  
                    FROM ITEMVENDA IV
                  JOIN PRODUTO P
                    ON IV.CDPRODUTO = P.CDPRODUTO
                        WHERE DTHRPEDIDO BETWEEN to_date(:DATA_INI, 'DD/MM/YYYY hh24:mi:ss') and to_date(:DATA_FIM, 'DD/MM/YYYY hh24:mi:ss')
                        ORDER BY NRSEQVENDA, NRSEQUITVEND
                ";
                
        $params = [
                    ':CDFILIAL'    => $subsidiaryCode,
                    ':DATA_INI'    => $dtInicial,
                    ':DATA_FIM'    => $dtFinal . ' ' . '23:59:59'
            ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
     public function getItensCombo ($subsidiaryCode, $cdCaixa, $nrSeqVenda, $nrSequItVend) {
        
        $sql = "SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, I.NRSEQUITVEND,
                       I.CDPRODUTO, P.NMPRODUTO, I.QTITVENDAEST, P.IDUTILRETPRODACAB
                    FROM ITEMVENDAEST I
                  JOIN PRODUTO P
                    ON I.CDPRODUTO = P.CDPRODUTO
                        WHERE I.CDFILIAL = :CDFILIAL
                            AND I.CDCAIXA = :CDCAIXA
                            AND I.NRSEQVENDA = :NRSEQVENDA
                            AND I.NRSEQUITVEND = :NRSEQUITVEND
                        ORDER BY I.NRSEQVENDA, I.NRSEQUITVEND
                ";
                
        $params = [
                    ':CDFILIAL'     => $subsidiaryCode,
                    ':CDCAIXA'      => $cdCaixa,
                    ':NRSEQVENDA'   => $nrSeqVenda,
                    ':NRSEQUITVEND' => $nrSequItVend
            ];
     
     return $this->connection->fetchAll($sql, $params);   
    }
    
    public function getComposicao($subsidiaryCode, $cdProduto) {
        $sqlLocal = "
            SELECT R.CDFILIAL, R.CDPRODUTO, R.CDSUBPRODLOC, P.NMPRODUTO, R.QTSUBPRODLOC, R.NRSEQPRODLOC 
                FROM ROTFABLOCAL R
                JOIN PRODUTO P ON R.CDSUBPRODLOC = P.CDPRODUTO
                    WHERE R.CDFILIAL = :CDFILIAL
                        AND R.CDPRODUTO = :CDPRODUTO
                        
            ORDER BY R.CDPRODUTO, R.NRSEQPRODLOC
        ";
        
        $paramsLocal = [
                    ':CDFILIAL'     => $subsidiaryCode,
                    ':CDPRODUTO'    => $cdProduto
            ];
        
        $localComp = $this->connection->fetchAll($sqlLocal, $paramsLocal);
        
        if(!empty($localComp))
            return $localComp;
            
        $sqlGeral = "
            SELECT R.CDPRODUTO, R.CDSUBPRODUTO, P.NMPRODUTO, R.QTSUBPRODROT, R.NRSEQPROD
                FROM ROTFABRICA R
                 JOIN PRODUTO P ON R.CDSUBPRODUTO = P.CDPRODUTO
                    WHERE R.CDPRODUTO = :CDPRODUTO
                    
            ORDER BY R.CDPRODUTO,R.NRSEQPROD
        ";
        
        $paramsGeral = [
                    ':CDPRODUTO'     => $cdProduto
            ];
        
        $geralComp = $this->connection->fetchAll($sqlGeral, $paramsGeral);
        
        return $geralComp;
    }
    
    public function getCodeProduct ($product) {
        
        $sql = "
            SELECT CDPRODINTE, SGUNIDADE, CDPRODUTO, CDARVPROD
              FROM PRODUTO PR
             WHERE CDPRODUTO  = :CDPRODUTO
                OR CDARVPROD  = :CDARVPROD
                OR CDPRODINTE = :CDPRODINTE
             AND CDPRODUTO = CDPRODESTO
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);
        

        $params = [
            ':CDPRODUTO'  => $productCode,
            ':CDARVPROD'  => $productCode,
            ':CDPRODINTE' => $productCode
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getProductFamily ($cdFilial, $product) {
        
        $sql = "
            SELECT CDFAMILISALD
              FROM FAMSALDOPROD
             WHERE CDFILIAL = :CDFILIAL
               AND CDPRODUTO  = :CDPRODUTO
        ";

        $params = [
            ':CDPRODUTO'  => $product,
            ':CDFILIAL'   => $cdFilial
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertProduct($cdFilial, $cdLoja, $cdCaixa, $cdProduto, $product, $nrSeqVenda, $nrSequitVend, $nrOrg, $cdFamilySald) {

        $sql = "
            INSERT INTO ITEMVENDA
             (CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND, CDFAMILISALD, QTPRODVEND, VRUNITVEND, VRDESITVEND, CDPRODUTO, VRUNITVENDCL,
              CDLOJA, VRACRITVEND, VRALIQIMPR, NRORDITCUPFIS, VRSUBSITVEND, NRORG, CDCAIXAPEDVEN, IDORIGPEDVEN, CDCLASFISC )
            VALUES
             (:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND, :CDFAMILISALD, :QTPRODVEND, :VRUNITVEND, :VRDESITVEND, :CDPRODUTO, :VRUNITVENDCL,
              :CDLOJA, :VRACRITVEND, :VRALIQIMPR, :NRORDITCUPFIS, :VRSUBSITVEND, :NRORG, :CDCAIXAPEDVEN, :IDORIGPEDVEN, :CDCLASFISC)
        ";
        
        $params = [
            ':CDFILIAL'      => $cdFilial,
            ':CDCAIXA'       => $cdCaixa,
            ':NRSEQVENDA'    => $nrSeqVenda,
            ':NRSEQUITVEND'  => str_pad($nrSequitVend, 3, "0", STR_PAD_LEFT),
            ':CDFAMILISALD'  => isset($cdFamilySald['CDFAMILISALD']) ? $cdFamilySald['CDFAMILISALD'] : NULL,
            ':QTPRODVEND'    => $product['quantity'],
            ':VRUNITVEND'    => str_replace('.', ',', $this->util->tofloat($product['price'])),
            ':VRDESITVEND'   => str_replace('.', ',', $this->util->tofloat($product['discount'])),
            ':CDPRODUTO'     => $cdProduto['CDPRODUTO'],
            ':VRUNITVENDCL'  => str_replace('.', ',', $this->util->tofloat($product['subsidizedValue'])),
            ':CDLOJA'        => $cdLoja,
            ':VRACRITVEND'   => str_replace('.', ',', $this->util->tofloat($product['addition'])),
            ':VRALIQIMPR'    => 0,
            ':NRORDITCUPFIS' => str_pad($nrSequitVend, 5, "0", STR_PAD_LEFT),
            ':VRSUBSITVEND'  => 0,
            ':NRORG'         => $nrOrg,
            ':CDCAIXAPEDVEN' => $cdCaixa,
            ':IDORIGPEDVEN'  => 'MOB',
            ':CDCLASFISC'    => $product['fiscalInfo']['ncm']
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function getProductTaxes($cdFilial, $cdProduto) {
        $sql = "
            SELECT A.CDPRODUTO, A.VRPEALIMPFIS, A.CDIMPOSTO, A.IDTPIMPOSFIS, A.IDINCIDEPISCOF, P.CDCLASFISC,
                   A.VRALIQIBPT, A.CDCSTICMS, A.CDCSTPISCOF, A.CDCFOPPFIS, A.CDREGRAIT, A.VRALIQIBPTES,
                   A.VRPERCREDUCAO, A.IDMODALBASECALC, A.CDREGRAITPC, A.CDCEST, A.VRALIQPIS, A.VRALIQCOFINS,
                   A.VRALIQFCP, A.VRPEALIMPFISEF, A.VRPERCREDUCAOEF, A.CDCBENEF, A.VRALIQICMSDES, A.CDNATUREC
            FROM ALIQIMPFIS A
                JOIN PRODUTO P
                    ON P.CDPRODUTO = A.CDPRODUTO
            WHERE A.CDFILIAL = :CDFILIAL
            AND A.CDPRODUTO  = :CDPRODUTO
        ";
        
        $params = [
            ':CDFILIAL'  => $cdFilial,
            ':CDPRODUTO' => $cdProduto
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getTaxesByProducts($cdFilial, $arrayProdutos) {
        
        $sql = "
            SELECT A.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.CDPRODINTE, A.VRPEALIMPFIS, A.CDIMPOSTO, A.IDTPIMPOSFIS, 
                   A.VRALIQIBPT, A.CDCSTICMS, A.CDCSTPISCOF, A.CDCFOPPFIS, A.CDREGRAIT, A.VRALIQIBPTES, A.IDINCIDEPISCOF,
                   A.VRPERCREDUCAO, A.IDMODALBASECALC, A.CDREGRAITPC, A.CDCEST, A.VRALIQPIS, A.VRALIQCOFINS, P.CDCLASFISC,
                   A.VRALIQFCP, A.VRPEALIMPFISEF, A.VRPERCREDUCAOEF, A.CDCBENEF, A.VRALIQICMSDES, A.CDNATUREC
            FROM ALIQIMPFIS A
                JOIN PRODUTO P
                    ON P.CDPRODUTO = A.CDPRODUTO
            WHERE A.CDFILIAL = :CDFILIAL
            AND A.CDPRODUTO IN (:ARR)
        ";
        
        $params = [
            ':CDFILIAL'  => $cdFilial,
            ':ARR'      => $arrayProdutos
        ];
        
        $type = [
            'ARR' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        ];
        
        return $this->connection->fetchAll($sql, $params, $type);
    }
    
    public function insertProductTaxes($cdFilial, $cdLoja, $cdCaixa, $cdProduto, $product, $nrSeqVenda, $nrSequitVend, $nrOrg, $productTaxes) {
        $sql = "
            INSERT INTO ITVENDAIMPOS
                (CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND,
                NRSEQITIMPOS, CDIMPOSTO, VRPEALPRODIT, VRIMPOPRODIT,
                CDIMPOSTOEX, CDCSTPRODI, CDCSTPRODPC, CDCFOPPROD,
                VRPERCOFINS, VRPERPIS, IDMODALBASECALC, VRBASECALCREDUZ,
                VRIMPOPRODREDUZ, VRBASECALCICMS, VRBCREDUZICMS, VRPRBCREDUICMS,
                VRIMPPIS, VRIMPCOFINS, VRTOTTRIBIBPT, VRBCPISCOFINS, NRORG, VRIMPICMBPIS, VRIMPICMBCOFINS)
            VALUES 
                (:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND,
                 :NRSEQITIMPOS, :CDIMPOSTO, :VRPEALPRODIT, :VRIMPOPRODIT,
                 :CDIMPOSTOEX, :CDCSTPRODI, :CDCSTPRODPC, :CDCFOPPROD,
                 :VRPERCOFINS, :VRPERPIS, :IDMODALBASECALC, :VRBASECALCREDUZ,
                 :VRIMPOPRODREDUZ, :VRBASECALCICMS, :VRBCREDUZICMS, :VRPRBCREDUICMS,
                 :VRIMPPIS, :VRIMPCOFINS, :VRTOTTRIBIBPT, :VRBCPISCOFINS, :NRORG, :VRIMPICMBPIS, :VRIMPICMBCOFINS)
        ";

        $params = [
            ':CDFILIAL'       => $cdFilial,
            ':CDCAIXA'        => $cdCaixa,
            ':NRSEQVENDA'     => $nrSeqVenda,
            ':NRSEQUITVEND'   => str_pad($nrSequitVend, 3, "0", STR_PAD_LEFT),
            ':NRSEQITIMPOS'   => '001',
            ':CDIMPOSTO'      => $productTaxes['CDIMPOSTO'],
            ':VRPEALPRODIT'   => str_replace('.', ',', $this->util->tofloat($productTaxes['VRPEALIMPFIS'])),
            ':VRIMPOPRODIT'   => str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['vrIcms'])),
            ':CDIMPOSTOEX'    => 0,
            ':CDCSTPRODI'     => $product['fiscalInfo']['cstIcms'],
            ':CDCSTPRODPC'    => $product['fiscalInfo']['cstPisCofins'],
            ':CDCFOPPROD'     => $product['fiscalInfo']['cfop'],
            ':VRPERCOFINS'    => str_replace('.', ',', $this->util->tofloat($productTaxes['VRALIQCOFINS'])),
            ':VRPERPIS'       => str_replace('.', ',', $this->util->tofloat($productTaxes['VRALIQPIS'])),
            ':IDMODALBASECALC'=> $productTaxes['IDMODALBASECALC'],
            ':VRBASECALCREDUZ'=> 0,
            ':VRIMPOPRODREDUZ'=> 0,
            ':VRBASECALCICMS' => str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['bcIcms'])),
            ':VRBCREDUZICMS'  => 0,
            ':VRPRBCREDUICMS' => 0,
            ':VRIMPPIS'       => str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['vrCofins'])),
            ':VRIMPCOFINS'    => str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['vrPis'])),
            ':VRTOTTRIBIBPT'  => 0, 
            ':VRBCPISCOFINS'  => str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['cstIcms'])), 
            ':NRORG'          => $nrOrg, 
            ':VRIMPICMBPIS'   => str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['vrPis'])), 
            ':VRIMPICMBCOFINS'=> str_replace('.', ',', $this->util->tofloat($product['fiscalInfo']['vrCofins']))
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function registerProductTaxes($branchDetails, $taxes, $cdProduto) {
        $sql = "
            INSERT INTO ALIQIMPFIS 
               (CDFILIAL, CDPRODUTO, VRPEALIMPFIS, CDIMPOSTO, IDTPIMPOSFIS, IDINCIDEPISCOF, NRORG, IDATIVO, VRALIQIBPT,
                CDCSTICMS, CDCSTPISCOF, CDCFOPPFIS, VRALIQIBPTES, VRPERCREDUCAO, IDMODALBASECALC, VRALIQPIS, VRALIQCOFINS,
                VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES, DTULTATUALIF, DTULTATU, DTINCLUSAO)
            VALUES
               (:CDFILIAL, :CDPRODUTO, :VRPEALIMPFIS, :CDIMPOSTO, :IDTPIMPOSFIS, :IDINCIDEPISCOF, :NRORG, :IDATIVO, :VRALIQIBPT,
                 :CDCSTICMS, :CDCSTPISCOF, :CDCFOPPFIS, :VRALIQIBPTES, :VRPERCREDUCAO, :IDMODALBASECALC, :VRALIQPIS, :VRALIQCOFINS,
                 :VRALIQFCP, :VRPEALIMPFISEF, :VRPERCREDUCAOEF, :CDCBENEF, :VRALIQICMSDES, TO_DATE(:DTULTATUALIF, 'DD/MM/YYYY hh24:mi:ss'),
                 TO_DATE(:DTULTATU, 'DD/MM/YYYY hh24:mi:ss'), TO_DATE(:DTINCLUSAO, 'DD/MM/YYYY hh24:mi:ss'))
        ";
        
        $dateTime = date('d/m/Y H:i:s');
        
        $params = [
            ':CDFILIAL'        => $branchDetails['CDFILIAL'],
            ':CDPRODUTO'       => $cdProduto,
            ':VRPEALIMPFIS'    => str_replace('.', ',', $this->util->tofloat($taxes['aliquot'])),
            ':CDIMPOSTO'       => $taxes['taxCode'],
            ':IDTPIMPOSFIS'    => $taxes['IDTPIMPOSFIS'],
            ':IDINCIDEPISCOF'  => $taxes['incidencePisCofins'] == 'S' ? 'S' : 'N',
            ':NRORG'           => $branchDetails['NRORG'],
            ':IDATIVO'         => 'S',
            ':VRALIQIBPT'      => str_replace('.', ',', $this->util->tofloat($taxes['aliqIBPTF'])),
            ':CDCSTICMS'       => $taxes['cstIcms'],
            ':CDCSTPISCOF'     => $taxes['cstPisCofins'],
            ':CDCFOPPFIS'      => $taxes['cfopCode'],
            ':VRALIQIBPTES'    => str_replace('.', ',', $this->util->tofloat($taxes['aliqIBPTE'])),
            ':VRPERCREDUCAO'   => str_replace('.', ',', $this->util->tofloat($taxes['icmsReductionPercentage'])),
            ':IDMODALBASECALC' => $taxes['icmsCalculationBasis'],
            ':VRALIQPIS'       => str_replace('.', ',', $this->util->tofloat($taxes['pisTaxRate'])),
            ':VRALIQCOFINS'    => str_replace('.', ',', $this->util->tofloat($taxes['cofinsTaxRate'])),
            ':VRALIQFCP'       => str_replace('.', ',', $this->util->tofloat($taxes['fcpTaxRate'])),
            ':VRPEALIMPFISEF'  => str_replace('.', ',', $this->util->tofloat($taxes['effectiveIcmsTaxRate'])),
            ':VRPERCREDUCAOEF' => str_replace('.', ',', $this->util->tofloat($taxes['effectiveIcmsRedPercentage'])),
            ':CDCBENEF'        => $taxes['fiscalBenefitCode'],
            ':VRALIQICMSDES'   => str_replace('.', ',', $this->util->tofloat($taxes['icmsExemptionRate'])),
            ':DTULTATUALIF'    => $dateTime,
            ':DTULTATU'        => $dateTime,
            ':DTINCLUSAO'      => $dateTime
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function UpdateProductTaxes($branchDetails, $taxes, $cdProduto) {
        $sql = "
            UPDATE ALIQIMPFIS 
                SET 
                    VRPEALIMPFIS    = :VRPEALIMPFIS,
                    CDIMPOSTO       = :CDIMPOSTO,
                    IDTPIMPOSFIS    = :IDTPIMPOSFIS,
                    IDINCIDEPISCOF  = :IDINCIDEPISCOF,
                    NRORG           = :NRORG,
                    IDATIVO         = :IDATIVO,
                    VRALIQIBPT      = :VRALIQIBPT,
                    CDCSTICMS       = :CDCSTICMS,
                    CDCSTPISCOF     = :CDCSTPISCOF,
                    CDCFOPPFIS      = :CDCFOPPFIS,
                    VRALIQIBPTES    = :VRALIQIBPTES,
                    VRPERCREDUCAO   = :VRPERCREDUCAO,
                    IDMODALBASECALC = :IDMODALBASECALC,
                    VRALIQPIS       = :VRALIQPIS,
                    VRALIQCOFINS    = :VRALIQCOFINS,
                    VRALIQFCP       = :VRALIQFCP,
                    VRPEALIMPFISEF  = :VRPEALIMPFISEF,
                    VRPERCREDUCAOEF = :VRPERCREDUCAOEF,
                    CDCBENEF        = :CDCBENEF,
                    VRALIQICMSDES   = :VRALIQICMSDES,
                    DTULTATUALIF    = TO_DATE(:DTULTATUALIF, 'DD/MM/YYYY hh24:mi:ss'),
                    DTULTATU        = TO_DATE(:DTULTATU, 'DD/MM/YYYY hh24:mi:ss')
            WHERE CDFILIAL = :CDFILIAL
             AND CDPRODUTO = :CDPRODUTO 
        ";
        
        $dateTime = date('d/m/Y H:i:s');
        
        $params = [
            ':CDFILIAL'        => $branchDetails['CDFILIAL'],
            ':CDPRODUTO'       => $cdProduto,
            ':VRPEALIMPFIS'    => str_replace('.', ',', $this->util->tofloat($taxes['aliquot'])),
            ':CDIMPOSTO'       => $taxes['taxCode'],
            ':IDTPIMPOSFIS'    => $taxes['IDTPIMPOSFIS'],
            ':IDINCIDEPISCOF'  => $taxes['incidencePisCofins'] == 'S' ? 'S' : 'N',
            ':NRORG'           => $branchDetails['NRORG'],
            ':IDATIVO'         => 'S',
            ':VRALIQIBPT'      => str_replace('.', ',', $this->util->tofloat($taxes['aliqIBPTF'])),
            ':CDCSTICMS'       => $taxes['cstIcms'],
            ':CDCSTPISCOF'     => $taxes['cstPisCofins'],
            ':CDCFOPPFIS'      => $taxes['cfopCode'],
            ':VRALIQIBPTES'    => str_replace('.', ',', $this->util->tofloat($taxes['aliqIBPTE'])),
            ':VRPERCREDUCAO'   => str_replace('.', ',', $this->util->tofloat($taxes['icmsReductionPercentage'])),
            ':IDMODALBASECALC' => $taxes['icmsCalculationBasis'],
            ':VRALIQPIS'       => str_replace('.', ',', $this->util->tofloat($taxes['pisTaxRate'])),
            ':VRALIQCOFINS'    => str_replace('.', ',', $this->util->tofloat($taxes['cofinsTaxRate'])),
            ':VRALIQFCP'       => str_replace('.', ',', $this->util->tofloat($taxes['fcpTaxRate'])),
            ':VRPEALIMPFISEF'  => str_replace('.', ',', $this->util->tofloat($taxes['effectiveIcmsTaxRate'])),
            ':VRPERCREDUCAOEF' => str_replace('.', ',', $this->util->tofloat($taxes['effectiveIcmsRedPercentage'])),
            ':CDCBENEF'        => $taxes['fiscalBenefitCode'],
            ':VRALIQICMSDES'   => str_replace('.', ',', $this->util->tofloat($taxes['icmsExemptionRate'])),
            ':DTULTATUALIF'     => $dateTime,
            ':DTULTATU'         => $dateTime
        ];

        $this->connection->executeQuery($sql, $params);
    }
}